#ifndef OVMAPI_H_
#define OVMAPI_H_

#ifdef WIN32

#ifdef OVMAPI_EXPORTS
#define OVMAPI_API __declspec(dllexport)
#else
#define OVMAPI_API __declspec(dllimport)
#endif

#else
#define OVMAPI_API
#define CALLBACK
#endif


#define OVMAPI_VERSION                  100

#define OVMAPI_MAX_KEYNAME_LEN          256
#define OVMAPI_MAX_KEYVALUE_LEN         8192
#define OVMAPI_MAX_PROC_NAME_LEN        256
#define OVMAPI_MAX_SUBSYS_NAME_LEN      64
#define OVMAPI_MAX_USER_DATA_LEN        4096

typedef void*                           OVMAPI_Handle;

typedef enum
{
    OVMAPI_EVT_SNAPSHOT             =   0x0001,
    OVMAPI_EVT_MIGRATE		        =   0x0002,
    OVMAPI_EVT_SHUTDOWN             =   0x0004,
    OVMAPI_EVT_RECONFIG             =   0x0008,
    OVMAPI_EVT_IP_ADDRESS           =   0x0010,
    OVMAPI_EVT_USER                 =   0x0020,
    OVMAPI_EVT_NEW_PARAM            =   0x0040,
	OVMAPI_EVT_SYSTEM               =   0x0080,
	OVMAPI_EVT_MAX                  =   0x0080

} OVMAPI_EventType;

typedef enum
{
	OVMAPI_EVT_PHASE_PRE			=	0x0000,
	OVMAPI_EVT_PHASE_IMMED			=	0x0001,
	OVMAPI_EVT_PHASE_POST			=	0x0002

} OVMAPI_EventPhase;

typedef enum
{
    OVMAPI_STATUS_SUCCESS           =   0x0000,
    OVMAPI_STATUS_UNSUCCESSFUL      =   0x0001,
    OVMAPI_STATUS_VERSION_MISMATCH  =   0x0002,
    OVMAPI_STATUS_CONNECT_FAIL      =   0x0003,
    OVMAPI_STATUS_FAIL_REGEVT       =   0x0004,
    OVMAPI_STATUS_TIMEOUT           =   0x0005,
    OVMAPI_STATUS_PENDING           =   0x0006,
    OVMAPI_STATUS_BUFFER_TOO_SMALL  =   0x0007,
    OVMAPI_STATUS_KEY_NOT_FOUND     =   0x0008,
    OVMAPI_STATUS_EVT_NOT_FOUND     =   0x0009,
    OVMAPI_STATUS_INVALID_ARG       =   0x000a,
    OVMAPI_STATUS_MAX               =   0x000a

} OVMAPI_Status;

typedef enum
{
    OVMAPI_FALSE                    =   0x0000,
    OVMAPI_TRUE                     =   0x0001

} OVMAPI_Boolean;

typedef enum
{
    OVMAPI_EVT_SEVERITY_DEBUG       =   0x0000,
    OVMAPI_EVT_SEVERITY_INFO        =   0x0001,
    OVMAPI_EVT_SEVERITY_WARNING     =   0x0002,
    OVMAPI_EVT_SEVERITY_ERROR       =   0x0003,
    OVMAPI_EVT_SEVERITY_CRITICAL    =   0x0004,
    OVMAPI_EVT_SEVERITY_SYSTEM      =   0x0005,
    OVMAPI_EVT_SEVERITY_MAX         =   0x0005

} OVMAPI_EventSeverity;


typedef struct _OVMAPI_EventShutdown
{
    OVMAPI_Boolean reboot;

} OVMAPI_EventShutdown;

typedef struct _OVMAPI_EventNewParam
{
    char keyName[OVMAPI_MAX_KEYNAME_LEN];

} OVMAPI_EventNewParam;

// The OVMAPI_EventUser structure is followed by the process name, then
// the subsystem, then the user data
typedef struct _OVMAPI_EventUser
{
    unsigned long eventNumber;
	unsigned short senderSubSystemOffset;
	unsigned short userDataOffset;

} OVMAPI_EventUser;

// The OVMAPI_EventSystem structure is followed by the event name, 
// then the process name, then the subsystem, then the system data
typedef struct _OVMAPI_EventSystem
{
	unsigned short senderProcessOffset;
	unsigned short senderSubSystemOffset;
	unsigned short systemDataOffset;

} OVMAPI_EventSystem;

typedef union _OVMAPI_EventData
{
    OVMAPI_EventShutdown shutdown;
    OVMAPI_EventUser user;
    OVMAPI_EventNewParam newParam;
	OVMAPI_EventSystem system;

} OVMAPI_EventData;

typedef struct _OVMAPI_Event
{
    unsigned long eventID;
    OVMAPI_EventType type;
	OVMAPI_EventPhase phase;
    OVMAPI_EventSeverity severity;
    unsigned short eventDataSize;
//    OVMAPI_EventData* eventData;
	char* eventData;

} OVMAPI_Event;

#define OVMAPI_EVENT_MAXSIZE		(sizeof(OVMAPI_Event))
#define OVMAPI_EVENT_DATA_MAXSIZE	(4096 - sizeof(OVMAPI_Event))


#ifdef __cplusplus
extern "C" {
#endif

/* OVMAPI_EventHandler
 *
 * A callback function that receives events from the VMAPI engine after a successful
 * call to OVMAPI_Register has been made, providing a valid callback handler.
 *
 * Parameters:
 *  OVMAPI_Event* :  This is the actual event structure that will be received each time
 *          callback is called.
 *
 *  PVOID : This is a pointer to a user defined context which was passed to the original
 *          call to OVMAPI_Register.
 *
 * Return Value:
 *  None
 *
*/
typedef OVMAPI_Status (CALLBACK *OVMAPI_EventHandler)(OVMAPI_Event*, void*);

/* OVMAPI_Register
 *
 * Call OVMAPI_Register to initialize the VMAPI and optionally register a callback.
 * OVMAPI_Register must be called before any other VMAPI calls can be made.
 *
 * Parameters:
 *  apiVersion (IN) :  Pass OVMAPI_VERSION, defined in the header.
 *
 *  eventHandler (IN, optional) : A function in the form of the OVMAPI_EventHandler to
 *          which event callbacks will be made.  Pass NULL if you do not wish to receive 
 *          VMAPI events in your application.
 *
 *  context (IN, optional) : An application defined context which will be sent to the 
 *          callback function upon delivery of each event.  This parameter is ignored if
 *          eventHandler is NULL.
 *
 *  session (OUT) : A pointer to a handle that the application must store.  This returned  
 *          handle will be used in all subsequent VMAPI calls.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_FAIL_REGEVT if the event handler failed to be registered
 *      OVMAPI_STATUS_CONNECT_FAIL if the API failed to connect to the VMAPI engine
 *      OVMAPI_STATUS_VERSION_MISMATCH if the header version does not match the binaries
*/
OVMAPI_API OVMAPI_Status OVMAPI_Register(int apiVersion, OVMAPI_EventHandler eventHandler, void* context, OVMAPI_Handle* session);

/* OVMAPI_UnRegister
 *
 * Call OVMAPI_UnRegister upon application termination, or when the VMAPI will no longer
 * be used by the application.
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL general failure.  Possibly the call to OVMAPI_Register was unsuccessful
*/
OVMAPI_API OVMAPI_Status OVMAPI_UnRegister(OVMAPI_Handle session);

/* OVMAPI_ParamGetValue
 *
 * Call OVMAPI_ParamGetValue to retrieve name-value pairs that are stored within the
 * VMAPI engine.
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  keyName (IN) : A NULL-terminated string representing the name for the value that will be retrieved.
 *
 *  keyValue (OUT) : An application allocated buffer which will receive the string value
 *          associated with the name passed as keyName.  The buffer should be large enough
 *          to hold the max sized value, defined as OVMAPI_MAX_KEYVALUE_LEN.
 *
 *  keyValueLen (IN, OUT) : A pointer to an integer which should be set to the size, in bytes,  
 *          of the keyValue buffer.  Upon return, keyValueLen will contain an integer value
 *          representing the size of the string returned in keyValue.
 *
 *  timeout (IN) : An integer to represent the amount of time, in milliseconds, that the call to 
 *          OVMAPI_ParamGetValue should block while waiting for the existence of the key.  Valid values
 *          are 0 - which will return instantly, any integer, or INFINITE - which will block until the 
 *          specified key exists.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the key does not exist.
 *      OVMAPI_STATUS_TIMEOUT if a timeout value was specified and the key did not exist within the 
 *          allotted time.
*/
OVMAPI_API OVMAPI_Status OVMAPI_ParamGetValue(OVMAPI_Handle session, const char* keyName, char* keyValue, unsigned long* keyValueLen, unsigned long timeout);

/* OVMAPI_ParamSetValue
 *
 * Call OVMAPI_ParamSetValue to create or modify a name-value pair
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  keyName (IN) : A NULL-terminated string representing the name for the value that will be set.  
 *          If the key already exists, the value will be overwritten.  Otherwise, a new key will be 
 *          created.
 *
 *  keyValue (IN) : a NULL-terminated string representing the value to be set.  The value should 
 *          not be larger than OVMAPI_MAX_KEYVALUE_LEN.
 *
 *  persist (IN) : Set to OVMAPI_True if the key should be retrievable.  Otherwise, the name-value
 *          pair is sent through the VMAPI engine but not stored.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
*/
OVMAPI_API OVMAPI_Status OVMAPI_ParamSetValue(OVMAPI_Handle session, const char* keyName, const char* keyValue, OVMAPI_Boolean persist);

/* OVMAPI_Subscribe
 *
 * Call OVMAPI_Subscribe to receive particular events in the application registered event handler
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  eventTypeMask (IN) : One or more OVMAPI_EventType values OR'ed together
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
*/
OVMAPI_API OVMAPI_Status OVMAPI_Subscribe(OVMAPI_Handle session, OVMAPI_EventType eventTypeMask);

/* OVMAPI_UnSubscribe
 *
 * Call OVMAPI_UnSubscribe to block particular events in the application registered event handler
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  eventTypeMask (IN) : One or more OVMAPI_EventType values OR'ed together
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
*/
OVMAPI_API OVMAPI_Status OVMAPI_UnSubscribe(OVMAPI_Handle session, OVMAPI_EventType eventTypeMask);

/* OVMAPI_EventComplete
 *
 * Events received in the application event handler can be completed by returning an appropriate
 * OVMAPI_Status status code from the event handler.  An application may choose to take action
 * upon a certain event, in which case it may return OVMAPI_STATUS_PENDING and then use 
 * OVMAPI_EventComplete to complete the event with a finalizing status code at some later time
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  event (IN) : The event that was received in the application event handler
 *
 *  status (IN) : The final status for which to complete this event
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
*/
OVMAPI_API OVMAPI_Status OVMAPI_EventComplete(OVMAPI_Handle session, OVMAPI_Event* event, OVMAPI_Status status);

/* OVMAPI_ParamGetValueSize
 *
 * Call OVMAPI_ParamGetValueSize to retrieve the size, in bytes, of a parameter value
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  keyName (IN) : A NULL-terminated string representing the name for the value length
 *          that will be retrieved.
 *
 * Return Value:
 *  The number of bytes required to fit the parameter value
*/
OVMAPI_API unsigned long OVMAPI_ParamGetValueSize(OVMAPI_Handle session, const char* keyName);

/* OVMAPI_GetSessionFileDescriptor
 *
 * Call OVMAPI_GetSessionFileDescriptor to retrieve the internal file descriptor for use in 
 * making special calls to the VMAPI driver.  Only processes that wish to receive events 
 * through poll/select should use this.  Standard read/write is forbidden.
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 * Return Value:
 *  The file descriptor for the VMAPI driver
*/
OVMAPI_API OVMAPI_Status OVMAPI_GetSessionFileDescriptors(OVMAPI_Handle session, int* sessionFDs, int* count);

/* OVMAPI_UserEventPublish
 *
 * Call OVMAPI_UserEventPublish to send application specific events to the management 
 * server and to other VMAPI enabled applications running on the same VM.
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  eventNumber (IN) :  An application defined identifier for this event type
 *
 *  severity (IN) :  An appropriate OVMAPI_EventSeverity for this event
 *
 *  processName (IN) :  The name of the sender process
 *
 *  subSystem (IN) :  The name of the sender subSystem
 *
 *  userData (IN) :  The payload for this user event.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
*/
OVMAPI_API OVMAPI_Status OVMAPI_UserEventPublish(OVMAPI_Handle session, unsigned long eventNumber, OVMAPI_EventSeverity severity, const char* processName, const char* subSystem, const char* userData);

/* OVMAPI_ParamGetAllNames
 *
 * OVMAPI_ParamGetAllNames should be used on application startup - after an event handler has
 * been registered, in order to browse for pre-existing parameters that may be of interest to 
 * the application.
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  keyNames (IN, OUT) :  An application supplied buffer that will be filled with keyNames of 
 *        existing parameters.  keyNames can be NULL, such that the first call can retrieve the 
 *        proper buffer size to be allocated by the application.
 *
 *  bufferSize (IN, OUT) :  As an IN param, the size of the buffer in bytes that was allocated and 
 *        passed in the keyNames parameter.  As an OUT param, the size of the data returned, or
 *        the size of the buffer required to fit all the data.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
 *      OVMAPI_STATUS_BUFFER_TOO_SMALL if the provided buffer is too small to fit the data
*/
OVMAPI_API OVMAPI_Status OVMAPI_ParamGetAllNames(OVMAPI_Handle session, char* keyNames, unsigned long* bufferSize);

/* OVMAPI_ParamGetCount
 *
 * Call OVMAPI_ParamGetCount to retrieve the number of existing parameters
 *
 * Parameters:
 *  session (IN) :  The handle that was returned by the call to OVMAPI_Register.
 *
 *  numParams (OUT) : The number of parameters stored in the VMAPI engine.
 *
 * Return Value:
 *  An OVMAPI_Status:
 *      OVMAPI_STATUS_SUCCESS if the call was successful.
 *      OVMAPI_STATUS_UNSUCCESSFUL if the call failed.
*/
OVMAPI_API OVMAPI_Status OVMAPI_ParamGetCount(OVMAPI_Handle session, unsigned long* numParams);

OVMAPI_API OVMAPI_Status OVMAPI_GetEvent(OVMAPI_Handle session, OVMAPI_Event** event);
OVMAPI_API OVMAPI_Status OVMAPI_EventPublish(OVMAPI_Handle session, OVMAPI_Event* event);
OVMAPI_API OVMAPI_Status OVMAPI_EventPost(OVMAPI_Handle session, OVMAPI_Event* event);
OVMAPI_API OVMAPI_Status OVMAPI_ParamDelete(OVMAPI_Handle session, const char* keyName);


#ifdef __cplusplus
}
#endif

#endif /* OVMAPI_H_ */
